using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace BuayaCorp
{
    public partial class Main : Form
    {
		FakeWebServer ws;
        public Main()
        {
            InitializeComponent();

			StartWebServer();
        }

		private void StartWebServer()
		{
			string response;
			using (StreamReader reader = new StreamReader("response.txt"))
			{
				response = reader.ReadToEnd();
			}

			ws = new FakeWebServer(80, response);
			ws.UrlReplaced += new OnUrlReplaced(ws_UrlReplaced);
		}

		void ws_UrlReplaced(string fakeUrl, string realUrl)
		{
			logger.AppendText(
				"Fake = " + fakeUrl + Environment.NewLine +
				"Real = " + realUrl + Environment.NewLine +
				"--" + Environment.NewLine + Environment.NewLine);
		}

		private void doXssButton_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(postUrl.Text.Trim()))
				return;

			if (!string.IsNullOrEmpty(hostname.Text.Trim()))
				Utils.Hostname = hostname.Text;

			responseText.Clear();
			doXssButton.Enabled = false;
			this.Cursor = Cursors.WaitCursor;

			worker.RunWorkerAsync();
		}

		private void Main_FormClosing(object sender, FormClosingEventArgs e)
		{
			ws.ForceShutdown();
		}

		private void worker_DoWork(object sender, DoWorkEventArgs e)
		{
			e.Result = Utils.SendXss(postUrl.Text.Trim(), trackbackUrl.Text.Trim(), useStoryTitle.Checked);
		}

		private void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (e.Error == null)
				responseText.Text = Regex.Replace(e.Result.ToString(), @"[\r\n]", Environment.NewLine);
			else
				responseText.Text = e.Error.Message;

			this.Cursor = Cursors.Default;
			doXssButton.Enabled = true;
		}

		private void bclink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			bclink.Show();
		}

    }
}