using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using System.Web;
using System.Collections.Specialized;

namespace BuayaCorp
{
	public delegate void OnUrlReplaced(string fakeUrl, string realUrl);
 
    public class FakeWebServer
    {
		public event OnUrlReplaced UrlReplaced;

        private const string URL_REPLACE = "{URL}";
        private static readonly Regex headerRegex = new Regex(@"^(GET|POST) /(.*?) (HTTP[^\s]+)", 
					RegexOptions.Compiled | RegexOptions.IgnoreCase);

		// Almacena las URLs verdaderas y falsas
		public static readonly Dictionary<string, string> FakeUrls;

        static FakeWebServer()
        {
            FakeUrls = new Dictionary<string, string>();
        }

        private TcpListener listener;
        string contents;
		int port;
		bool halted;

        public FakeWebServer(int port, string responseContents)
        {
            this.contents = responseContents;
			this.port = port;

			// 'Escuchar' en cualquier direccin
            listener = new TcpListener(IPAddress.Any, port);
            listener.Start();

            Thread t = new Thread(delegate()
            {
                AcceptClients();
            });
            t.Start();
        }

		public void AcceptClients()
		{
			while (true)
			{
				using (TcpClient client = listener.AcceptTcpClient())
				{
					if (client.Connected) // Nuevo cliente
					{
						// Leer los datos enviados
						NetworkStream stream = client.GetStream();
						byte[] data = new byte[1024];

						stream.Read(data, 0, data.Length);

						string request = Encoding.UTF8.GetString(data);
						
						// Slo tomar en cuenta los datos presentes en el QueryString
						// Obtener la versin del protocolo y la URL del 'Request'
						Match match = headerRegex.Match(request);

						string httpVersion = match.Groups[3].Value;

						string fakeUrl = match.Groups[2].Value;
						string realUrl = string.Empty;

						if (fakeUrl == "shutdown.xss")
						{
							SendHeaders(httpVersion, null, 0, "200 OK", client);
							break;
						}

						if (!FakeUrls.TryGetValue(fakeUrl, out realUrl))
						{
							fakeUrl = fakeUrl.Replace("\\", "");
							FakeUrls.TryGetValue(fakeUrl, out realUrl);
						}
						
						
						System.Diagnostics.Debug.WriteLine("Query String: " + match.Groups[2].Value);

						// Reemplazar las URLs
						if (!string.IsNullOrEmpty(realUrl) && !string.IsNullOrEmpty(contents))
						{
							contents = contents.Replace(URL_REPLACE, realUrl);
							contents += string.Format("<a href=\"{0}\">{0}</a>", realUrl.Replace(".php", ""));
							if (UrlReplaced != null)
								UrlReplaced(fakeUrl, realUrl);
						}

						// Enviar las cabeceras necesarias y el contenido
						SendHeaders(httpVersion, null, contents.Length, "200 OK", client);
						SendToBrowser(Encoding.UTF8.GetBytes(contents), client);
					}
				}
			}
			listener.Stop();
			halted = true;
		}

        public void SendHeaders(string httpVersion, string mimeHeader, int totalBytes, string statusCode, TcpClient tcpClient)
        {
            StringBuilder responseBuilder = new StringBuilder();

            if (string.IsNullOrEmpty(mimeHeader))
                mimeHeader = "text/html";

            responseBuilder.Append(httpVersion);
            responseBuilder.Append(' ');
            responseBuilder.AppendLine(statusCode);
            responseBuilder.AppendLine("Server: Fake Web Server");
            responseBuilder.Append("Content-Type: ");
            responseBuilder.AppendLine(mimeHeader);
            responseBuilder.AppendLine("Accept-Ranges: bytes");
            responseBuilder.Append("Content-Length: ");
            responseBuilder.AppendLine(totalBytes.ToString());
            responseBuilder.AppendLine("");

            Byte[] bSendData = Encoding.UTF8.GetBytes(responseBuilder.ToString());
            SendToBrowser(bSendData, tcpClient);

			//System.Diagnostics.Debug.WriteLine("Total Bytes : " + totalBytes.ToString());
        }

        public void SendToBrowser(Byte[] data, TcpClient tcpClient)
        {
            if (tcpClient.Connected)
            {
                NetworkStream stream = tcpClient.GetStream();

                stream.Write(data, 0, data.Length);
                stream.Flush();
            }
            else
            {
                System.Diagnostics.Debug.WriteLine("Connection Dropped....");
            }

        }

		// Lame shutdown
		public void ForceShutdown()
		{
			if (!halted)
			{
				try
				{
					using (WebClient client = new WebClient())
					{
						client.DownloadString("http://localhost:" + port + "/shutdown.xss");
					}
				}
				catch{}
			}
		}
	}
}
